# -*- coding: utf-8 -*-
# FOGLAMP_BEGIN
# See: https://foglamp-foglamp-documentation.readthedocs-hosted.com
# FOGLAMP_END

""" Plugin module which converts attributes into individual assets """

import logging
import traceback
import copy

from foglamp.common import logger
import filter_ingest

_LOGGER = logger.setup(__name__, level=logging.INFO)


_DEFAULT_CONFIG = {
    'plugin': {        # mandatory  filter
        'description': 'filter that converts attribute into individual assets',
        'type': 'string',
        'default': 'asset_split',
        'readonly': 'true'
    },
    'enable': {    # recommended filter
        'description': 'Enable asset split filter plugin',
        'type': 'boolean',
        'default': 'false',
        'displayName': 'Enabled',
        'order': "1"
    },
    'prefix': {
        'description': 'Static prefix before [asset].[attribute]',
        'type': 'string',
        'default': '',
        'displayName': 'Prefix',
        'order': '2'
    }
}


def plugin_info():
    """ Returns information about the plugin.
    Args:
    Returns:
        dict: plugin information
    Raises:
    """
    return {
        'name': 'AssetSplit',
        'version': '2.0.1',
        'mode': 'none',
        'type': 'filter',
        'interface': '1.0',
        'config': _DEFAULT_CONFIG
    }


def plugin_init(config, ingest_ref, callback):
    """ Initialise the plugin.
    Args:
        config: JSON configuration document for the plugin configuration category
        ingest_ref: filter ingest reference
        callback:   filter callback
    Returns:
        data: JSON object to be used in future calls to the plugin
    Raises:
    """
    _config = copy.deepcopy(config)
    _config['ingestRef'] = ingest_ref
    _config['callback'] = callback
    return _config


def plugin_ingest(handle, data):
    """ plugin_ingest -- log data we receive """

    try:
        if type(data) == dict:
            data = [data]
        data_split = []
        for entry in data:
            base = copy.deepcopy(entry)
            for reading_key in entry['readings']:
                split_reading = copy.deepcopy(base)
                asset_attr_name = "{}{}.{}".format(handle['prefix']['value'], split_reading['asset'], reading_key)
                split_reading['readings'] = {asset_attr_name: entry['readings'][reading_key]}
                split_reading['asset'] = asset_attr_name
                data_split.append(split_reading)
        filter_ingest.filter_ingest_callback(handle['callback'], handle['ingestRef'], data_split)
    except Exception:
        _LOGGER.error("AssetSplit writer exception {}".format(traceback.format_exc()))
        raise


def plugin_reconfigure(handle, new_config):
    """ Reconfigures the plugin
    Args:
        handle: handle returned by the plugin initialisation call
        new_config: JSON object representing the new configuration category for the category
    Returns:
        new_handle: new handle to be used in the future calls
    """

    _LOGGER.info("Old config for  plugin {} \n new config {}".format(handle, new_config))

    new_handle = copy.deepcopy(new_config)
    new_handle['ingestRef'] = handle['ingestRef']
    new_handle['callback'] = handle['callback']
    return new_handle


def plugin_shutdown(handle):
    """ Shut down the plugin.
    Args:
        handle: JSON configuration document for the plugin configuration category
    Returns:
        data: JSON object to be used in future calls to the plugin
    Raises:
    """
    handle['callback'] = None
    handle['ingestRef'] = None
    _LOGGER.info("AssetSplit filter plugin shutdown.")
